-------------------------------------------------------------------
-- Date 		: 04-May-2012
-- Database		: DB2
-- Description 	: ISAMESSO Sync Data Housekeeping Maintenance Script
-- IMS Version	: 8.1.x / 8.2.x
--
--  Syntax : sp_purgeDelSyncDataOlderThan '<del mode>', <days> 
--  * <del mode> - Mandatory, text, case sensitive  
--				 - ''ABOUT'' display script information 
--				 - ''DELETE'' to perform actual removal 
--				 - ''HELP'' display script syntax and example 
--				 - ''SIMULATE'' to simulate deletion without actual removal 
--  * <days> - Mandatory, numeric  
--           - Enter 0 to delete all 
--
-- Example : 
--  * call sp_purgeDelSyncDataOlderThan('SIMULATE',30)  - Simulate purging of deleted sync data older than 30 days without performing actual removal
--  * call sp_purgeDelSyncDataOlderThan('DELETE',14)    - Purges deleted sync data older than 2 weeks (14 days) from database
--  * call sp_purgeDelSyncDataOlderThan('DELETE',0)     - Purges all deleted sync data from database
--
-- user, system and machine-scoped Sync Data 
-------------------------------------------------------------------
--
-- DROP PROCEDURE sp_purgeDelSyncDataOlderThan;
--
-- call sp_purgeDelSyncDataOlderThan('ABOUT',30);
-- call sp_purgeDelSyncDataOlderThan('SIMULATE',30);
-- call sp_purgeDelSyncDataOlderThan('DELETE',30);
-- call sp_purgeDelSyncDataOlderThan('HELP',30);
-- call sp_purgeDelSyncDataOlderThan('',0);
--
-- Tables affected : 
--  IMSTemplatedSyncRepo 
--

CREATE PROCEDURE sp_purgeDelSyncDataOlderThan
(
IN IN_operationCmd VARCHAR(16), 
IN IN_days INTEGER
)
DYNAMIC RESULT SETS 1
MODIFIES SQL DATA
LANGUAGE SQL 
BEGIN
	
	DECLARE @info_script_ver varchar(8) DEFAULT '1.0.0';
	DECLARE @info_start_timestamp TIMESTAMP;
	DECLARE @info_end_timestamp TIMESTAMP;
	DECLARE @info_elapsed_time varchar(32);
	
	DECLARE @var_tsRetainDate TIMESTAMP;
	
	DECLARE @var_del_tmpCnt INTEGER DEFAULT 0;
	DECLARE @val_temp_int INTEGER DEFAULT 0;
	DECLARE @var_iTemplatedSyncRepoExist INTEGER DEFAULT 0;
	
	DECLARE @var_del_mode varchar(16) DEFAULT '';	
	DECLARE @var_backup_schema varchar(32) DEFAULT NULL;
 	DECLARE @var_ims_schema varchar(32) DEFAULT NULL;
 
	DECLARE @val_message varchar(128) DEFAULT NULL;
	DECLARE @exception_msg varchar(128) DEFAULT NULL;
	DECLARE @var_del_result INTEGER DEFAULT 0;
	
	-- result output
	DECLARE @cursor_result_output CURSOR WITH RETURN TO CLIENT FOR
	SELECT msg from
	(
		SELECT 0 id, ('command : sp_purgeDelSyncDataOlderThan ('''|| IN_operationCmd || ''',''' || TRIM(CHAR(IN_days)) || ')' ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  2 id, ('  Input  > Delete mode : ' || IN_operationCmd) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  3 id, ('  Input  > Days : ' || TRIM(CHAR(IN_days))) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  4 id, ('  Output > Purge records older than (YYYYMMDD) : ' || TO_CHAR(@var_tsRetainDate,'YYYYMMDD')) msg FROM SYSIBM.SYSDUMMY1
		
		union SELECT  5 id, ('  Output > - ' || @var_del_mode ||' IMSTemplatedSyncRepo : '|| CHAR(@var_del_result)) msg FROM SYSIBM.SYSDUMMY1

		union SELECT  10 id, ('  Output > Start time : ' || CHAR(date(@info_start_timestamp)) || ' ' || CHAR(time(@info_start_timestamp)) ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  11 id, ('  Output > End time : ' || CHAR(date(@info_end_timestamp)) || ' ' || CHAR(time(@info_end_timestamp)) ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  12 id, ('  Output > Total elapsed time : ' || @info_elapsed_time) msg FROM SYSIBM.SYSDUMMY1
		
	) order by id;
	

	DECLARE @cursor_about CURSOR WITH RETURN TO CLIENT FOR
	SELECT msg from
	(
		SELECT  1 id, (' ISAMESSO Sync Data Housekeeping Maintenance Script ' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  2 id, ('  - Database Name    : ' || CURRENT SERVER )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  3 id, ('  - Schema Name      : ' || CURRENT SCHEMA )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  4 id, ('  - Script Version   : ' || @info_script_ver )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  5 id, ('  - Database Version : ' || (SELECT service_level FROM TABLE (sysproc.env_get_inst_info())) )msg FROM SYSIBM.SYSDUMMY1
	) order by id;

	
	DECLARE @cursor_syntax CURSOR WITH RETURN TO CLIENT FOR
	SELECT msg from
	(
		SELECT 0 id, ('command : sp_purgeDelSyncDataOlderThan ('''|| IN_operationCmd || ''',''' || TRIM(CHAR(IN_days)) || ')' ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  1 id,  CASE WHEN @val_message IS NULL OR TRIM(@val_message) = '' THEN '' ELSE ' * ' || @val_message END msg FROM SYSIBM.SYSDUMMY1
		union SELECT  2 id, ('Syntax : sp_purgeDelSyncDataOlderThan(''<del mode>'', <days> )' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  6 id, (' * <del mode> - Mandatory, text, case sensitive ' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  7 id, ('  			- ''ABOUT'' display script information' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  8 id, ('  			- ''DELETE'' to perform actual delete' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  9 id, ('  			- ''HELP'' display script syntax and example' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  10 id, ('  			- ''SIMULATE'' to simulate deletion without actual delete' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  11 id, (' * <days> 	- Mandatory, numeric' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  12 id, ('  			- Enter 0 to delete all' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  20 id, (' ') msg FROM SYSIBM.SYSDUMMY1
		union SELECT  21 id, (' Example : ') msg FROM SYSIBM.SYSDUMMY1 
		union SELECT  22 id, (' * sp_purgeDelSyncDataOlderThan(''SIMULATE'',30) ') msg FROM SYSIBM.SYSDUMMY1
		union SELECT  23 id, ('   - Simulate purging of deleted sync data older than 30 days without performing actual removal')msg FROM SYSIBM.SYSDUMMY1
		union SELECT  25 id, (' * sp_purgeDelSyncDataOlderThan(''DELETE'',14) ') msg FROM SYSIBM.SYSDUMMY1
		union SELECT  26 id, ('   - Purges deleted sync data older than 2 weeks (14 days) from database') msg FROM SYSIBM.SYSDUMMY1
		union SELECT  28 id, (' * sp_purgeDelSyncDataOlderThan(''DELETE'',0) ') msg FROM SYSIBM.SYSDUMMY1
		union SELECT  29 id, ('   - Purges all deleted sync data from database') msg FROM SYSIBM.SYSDUMMY1		
		
		
	) order by id;
	
	-- Handle SQLException
	DECLARE EXIT HANDLER FOR SQLEXCEPTION
	BEGIN
		GET DIAGNOSTICS EXCEPTION 1 @exception_msg = MESSAGE_TEXT;
		
		IF @val_message IS NULL THEN
			SET @val_message = @exception_msg;
		END IF;
		
		IF IN_operationCmd = 'ABOUT' THEN
			IF TRIM(@val_message) = '' THEN
				OPEN @cursor_about;
			ELSE
				OPEN @cursor_syntax;
			END IF;
		ELSEIF IN_operationCmd = 'HELP' THEN
			OPEN @cursor_syntax;
		ELSE
			OPEN @cursor_syntax;
		END IF;
	END;
	
	--SET @var_ims_schema = IN_imsSchema;
	
	-- capture start time
	SELECT CURRENT TIMESTAMP INTO @info_start_timestamp FROM SYSIBM.SYSDUMMY1;
	

	IF @var_ims_schema IS NULL THEN
		SET @var_ims_schema = '';
	END IF;
	
	-- check IMS Schema name input
	IF TRIM(@var_ims_schema) <> ''  THEN
		-- Capture current schema
		SELECT CURRENT SCHEMA INTO @var_backup_schema FROM SYSIBM.SYSDUMMY1;
		
		-- Set current schema to IMS schema
		SET CURRENT SCHEMA = @var_ims_schema;
	END IF;
	
	-- Check is IMS schema
	SELECT DISTINCT COUNT(TABNAME) INTO @val_temp_int FROM SYSCAT.TABLES 
		WHERE UPPER(TABNAME) = 'IMSTEMPLATEDSYNCREPO' AND TABSCHEMA = CURRENT SCHEMA;
	
	-- Valid IMS schema will have at least 4 IMS log tables
	IF @val_temp_int <> 1 THEN
		SET @val_message = 'Error : Invalid ISAMESSO schema - ' || CURRENT SCHEMA;
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	END IF;
	
	
	-- ## INPUT VALIDATION
	IF IN_operationCmd = 'ABOUT' THEN
		-- force SQLException to exit with ABOUT info
		SET @val_message = '';
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	ELSEIF IN_operationCmd = 'HELP' THEN
		-- force SQLException to exit with HELP
		SET @val_message = '';
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
		
	ELSEIF IN_operationCmd = 'SIMULATE' THEN
	 	-- SIMULATE
		-- do nothing
		SET @var_del_mode  = 'calculating';
	ELSEIF IN_operationCmd = 'DELETE' THEN
		-- delete
		-- do nothing
		SET @var_del_mode  = 'deleted';
	ELSE
		-- syntax error , force SQLException to exit
		SET @val_message = 'Error : Invalid delete mode.';
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	END IF;			
	
		
	-- Calc retain date
	SELECT (CURRENT TIMESTAMP - (IN_days DAY)) INTO @var_tsRetainDate FROM SYSIBM.SYSDUMMY1;
	

	IF IN_operationCmd = 'DELETE' THEN
		SET @var_del_tmpCnt = 1;
		WHILE @var_del_tmpCnt>0 DO
			--
			DELETE FROM IMSTemplatedSyncRepo WHERE (attrName, entityTypeId, entityId, keyValue, locale) IN 
				(SELECT attrName, entityTypeId, entityId, keyValue, locale 
					FROM IMSTemplatedSyncRepo WHERE isDeleted = 1 AND updateTimeStamp < @var_tsRetainDate 
						FETCH FIRST 10000 ROWS ONLY);
			--
			GET DIAGNOSTICS @var_del_tmpCnt = ROW_COUNT;
			SET @var_del_result = @var_del_result + @var_del_tmpCnt;
			COMMIT;			
		END WHILE;
	ELSE
		SELECT COUNT(*) INTO @var_del_result FROM IMSTemplatedSyncRepo WHERE isDeleted = 1 AND updateTimeStamp < @var_tsRetainDate;
	END IF;
	--
	
	-- Reset schema to previous default schema
	IF @var_backup_schema IS NOT NULL THEN
		SET CURRENT SCHEMA = @var_backup_schema;
	END IF;
	
	-- capture end time
	SELECT CURRENT TIMESTAMP INTO @info_end_timestamp FROM SYSIBM.SYSDUMMY1;
		
	--calc elapsed time
	SELECT 	
		CASE WHEN DAY(time_diff)>0 THEN TRIM(CHAR(DAY(time_diff))) || 'd ' ELSE '' END
		|| CASE WHEN HOUR(time_diff)>0 THEN TRIM(CHAR(HOUR(time_diff))) || 'h ' ELSE '' END
		|| CASE WHEN MINUTE(time_diff)>0 THEN TRIM(CHAR(MINUTE(time_diff))) || 'm ' ELSE '' END
		|| TRIM(CHAR(SECOND(time_diff))) || 's ' 
		|| SUBSTR(TRIM(CHAR(MICROSECOND(CURRENT TIMESTAMP))),1,4) || 'ms' 
	INTO @info_elapsed_time 
		FROM ( SELECT (@info_end_timestamp - @info_start_timestamp) time_diff FROM SYSIBM.SYSDUMMY1);

	OPEN @cursor_result_output;
	
END
#